


#include "core.h"

void numAdd(int in1[], int in2[], int output[], int in1len, int in2len)
{
	// Assumes in1 is longer or at least same length as in2
	int carry = 0;

	for (int i = 0; i < in1len; i++)
	{
		if (i < in2len)
		{
			output[i] = in1[i] + in2[i] + carry;

			if (output[i] > 9999)
			{
				output[i] = output[i] - 10000;
				carry = 1;
			}
			else
				carry = 0;
		}
		else
		{
			output[i] = in1[i];
			// If there is a carry, the above isn't true

			if (carry == 1)
			{
				output[i] = output[i] + 1;
				// Turn carry back into 0 now that it is gone
				carry = 0;

				// If the number is 9999 and there was a carry, we have a special case
				if (output[i] == 10000)
				{
					output[i] = 0;
					// Carry is now 1 again
					carry = 1;
				}
			}
		}
	}

	if (carry == 1)
	{
		output[in1len + 1] = 1;
	}
}

void numSub(int in1[], int in2[], int output[], int in1len, int in2len)
{
	// Assumes in1 is larger or equal to in2
	int negCarry = 0;

	for (int i = 0; i < in1len; i++)
	{
		if (i < in2len)
		{
			if (in1[i] - negCarry < in2[i])
			{
				output[i] = in1[i] - negCarry - in2[i];
				negCarry = 0;
			}
			else
			{
				output[i] = 10000 + in1[i] - negCarry - in2[i];
				negCarry = 1;
			}
		}
		else
		{
			if (in1[i] == 0 && negCarry == 1)
			{
				output[i] = 9;
			}
			else
			{
				output[i] = in1[i] - negCarry;
				negCarry = 0;
			}
		}
	}
}

int *numMul(int in1[], int in2[], int* ansLen, int in1len, int in2len)
{
	*ansLen = in1len + in2len;
	// An array with in1len + in2len ints
	int *output = calloc(*ansLen, sizeof(int));

	// Doesn't matter which input is bigger
	int carry = 0;

	// In honor of Intel's processor family ;)
	int i3;
	int i5;
	for (i3 = 0; i3 < in2len; i3++)
	{
		for (i5 = 0; i5 < in1len; i5++)
		{
			int unmodded = carry + output[i3 + i5] + in1[i5] * in2[i3];
			carry = unmodded / 10000;
			output[i3 + i5] = unmodded - carry * 10000;
		}

		// Since i5 is already one higher than usual, nothing needs to be added to i3 + i5 - it is already the next term
		output[i3 + i5] = carry;

		carry = 0;
	}

	if (output[(*ansLen) - 1] == 0)
	{
		(*ansLen)--;
	}

	return output;
}

// Now obselete
int *numPow(int in1[], int power, int* ansLen, int in1len)
{
	int *output = 0;

	int *base = calloc(in1len, sizeof(int));
	for (int i = 0; i < in1len; i++)
		base[i] = in1[i];

	// Previous answer array number of elements
	int prevAnsLen = in1len;
	*ansLen = prevAnsLen + in1len;

	for (int i = 1; i < power; i++)
	{
		free(output);
		output = numMul(in1, base, ansLen, prevAnsLen, in1len);

		in1 = realloc(in1, *ansLen << 2);
		for (int j = 0; j < *ansLen; j++)
			in1[j] = output[j];

		prevAnsLen = *ansLen;
		*ansLen = *ansLen + in1len;
	}

	*ansLen = prevAnsLen;

	// Because at the end of the for loop an in1len is added without being used
	return output;
}

int *fastNumPow(int in1[], int power, int* ansLen, int in1len)
{
	int *ans = calloc(1, sizeof(int));
	ans[0] = 1;
	*ansLen = 1;

	int returnLen = 0;
	int *in1Copy;
	int *ansCopy;

	while (power > 0)
	{
		if ((power & 1) == 1)
		{
			ansCopy = ans;
			ans = numMul(ans, in1, &returnLen, *ansLen, in1len);
			free(ansCopy);
			*ansLen = returnLen;
		}

		in1Copy = in1;
		in1 = numMul(in1, in1, &returnLen, in1len, in1len);
		free(in1Copy);
		in1len = returnLen;

		power = power >> 1;
	}

	return ans;
}

int *numSum(int in[], int in1len, int* sumLen)
{
	int *sum = calloc(3, sizeof(int));
	int *perGroupSum = calloc(1, sizeof(int));
	int temp, temp2;

	for (int i = 0; i < in1len; i++)
	{
		temp = in[i] / 100;
		temp2 = in[i] - (100 * temp);

		perGroupSum[0] = (temp % 10) + (temp / 10) + (temp2 % 10) + (temp2 / 10);
		numAdd(sum, perGroupSum, sum, 3, 1);
	}

	*sumLen = 3;
	removeZeroes(sum, sumLen);
	return sum;
}

// Is zeroes spelled correctly?
void removeZeroes(int input[], int* inLen)
{
	for (; input[(*inLen - 1)] == 0; (*inLen)--)
	{
	}
}


